"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryTransformSchemaPipe = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const ajv_1 = require("ajv");
const helper_1 = require("../../../helper");
const types_1 = require("../../../types");
class QueryTransformSchemaPipe {
    constructor(repository, ajvCall) {
        this.repository = repository;
        this.ajvCall = ajvCall;
        this.allowPropsField = [];
        this.allowPropsFilter = [];
        this.allowPropsSort = [];
        this.allowPropsRelationFilter = {};
        this.allowPropsRelationSort = {};
        const schemaName = (0, helper_1.getEntityName)(this.repository.target);
        this.validateFunction = this.ajvCall.getSchema(`transformQuerySchema-${schemaName}`);
        this.allowPropsFilter = Object.keys(this.validateFunction.schema['$defs']['filterTarget'].properties);
        this.allowPropsSort = Object.keys(this.validateFunction.schema['$defs']['sortDefs'].properties.target
            .properties);
        this.allowPropsField = Object.keys(this.validateFunction.schema['$defs']['fieldsDefs'].properties);
        this.allowPropsRelationFilter = this.allowPropsRelationSort = Object.keys(this.validateFunction.schema['$defs'].filterRelation.properties).reduce((acum, item) => {
            acum[item] = Object.keys(this.validateFunction.schema['$defs'].filterRelation.properties[item]
                .properties);
            return acum;
        }, {});
        this.allowPropsRelationSort = Object.keys(this.validateFunction.schema['$defs']['sortDefs'].properties).reduce((acum, item) => {
            acum[item] = Object.keys(this.validateFunction.schema['$defs']['sortDefs'].properties[item]
                .properties);
            return acum;
        }, {});
    }
    static inject(pip) {
        (0, common_1.Inject)(ajv_1.default)(pip, 'ajvCall', 1);
    }
    transform(value) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const validate = this.validateFunction(value);
            if (!validate) {
                throw new common_1.BadRequestException(this.getErrors(this.validateFunction.errors));
            }
            return value;
        });
    }
    getErrors(errors) {
        const errorResult = [];
        for (const error of errors) {
            const parameterParts = error.instancePath
                .split('/')
                .filter((value) => value !== '');
            const fields = parameterParts[0];
            let detailMsg = '';
            switch (fields) {
                case 'fields': {
                    switch (error.keyword) {
                        case 'minProperties':
                            detailMsg = error.message;
                            break;
                        case 'enum':
                            detailMsg = `${error.message}. The allowed values are: "${error.params.allowedValues.join(',')}"`;
                            break;
                        case 'additionalProperties':
                            detailMsg = `${error.message}. The allowed properties are: "${this.allowPropsField.join(',')}"`;
                            break;
                    }
                    break;
                }
                case 'filter': {
                    if (error.instancePath === '/filter/target' &&
                        error.schemaPath === '#/additionalProperties') {
                        detailMsg = `${error.message}. The additionalProperty properties is: "${error.params.additionalProperty}"`;
                    }
                    if (error.keyword === 'maxProperties') {
                        detailMsg = `${error.message}. Only one operator allow for field`;
                    }
                    if ((error.keyword === 'minProperties' || error.keyword === 'type') &&
                        error.message !== 'must be null') {
                        detailMsg = error.message;
                    }
                    if (error.schemaPath.indexOf('target/additionalProperties') > -1) {
                        detailMsg = `${error.message}. The allowed properties are: "${this.allowPropsFilter.join(',')}"`;
                    }
                    if (error.schemaPath.indexOf('relation') > -1) {
                        if (error.schemaPath.indexOf('relation/additionalProperties') > -1) {
                            detailMsg = `${error.message}. The allowed properties are: "${Object.keys(this.allowPropsRelationFilter).join(',')}"`;
                        }
                        else if (error.keyword === 'additionalProperties') {
                            const relationName = error.instancePath.split('/').pop();
                            detailMsg = `${error.message}. The allowed properties for "${relationName}" are: "${this.allowPropsRelationFilter[relationName].join(',')}"`;
                        }
                    }
                    if (error.schemaPath.indexOf('operandForRelationProps') > -1) {
                        detailMsg = `Allowed only '${types_1.FilterOperand.eq}' and '${types_1.FilterOperand.ne}' operands and value only 'null'`;
                    }
                    if (error.schemaPath.indexOf('operandForArrayProps') > -1) {
                        detailMsg = `Incorrect operand '${error.params.additionalProperty}' in array field `;
                    }
                    if (error.schemaPath.indexOf('operand/additionalProperties') > -1) {
                        detailMsg = `Incorrect operand '${error.params.additionalProperty}' in non-array field `;
                    }
                    break;
                }
                case 'include': {
                    if (error.keyword === 'minItems') {
                        detailMsg = error.message;
                    }
                    if (error.keyword === 'enum') {
                        detailMsg = `${error.message}. The allowed values are: "${error.params.allowedValues.join(',')}"`;
                    }
                    break;
                }
                case 'sort': {
                    if (error.schemaPath.indexOf('targetResource/additionalProperties') > -1) {
                        detailMsg = `${error.message}. The allowed properties are: "${this.allowPropsSort.join(',')}"`;
                    }
                    if (error.schemaPath === '#/additionalProperties') {
                        detailMsg = `${error.message}. The allowed properties are: "${Object.keys(this.allowPropsRelationSort).join(',')}"`;
                    }
                    if (error.schemaPath === '#/properties/target/additionalProperties') {
                        detailMsg = `${error.message}. The allowed properties are: "${this.allowPropsSort.join(',')}"`;
                    }
                    if (error.keyword === 'enum') {
                        detailMsg = `${error.message}. The allowed values are: "${error.params.allowedValues.join(',')}"`;
                    }
                    if (error.keyword === 'minProperties') {
                        detailMsg = `${error.message}`;
                    }
                    break;
                }
            }
            if (detailMsg) {
                errorResult.push({
                    source: {
                        parameter: error.instancePath,
                    },
                    detail: detailMsg[0].toUpperCase() + detailMsg.slice(1),
                });
            }
        }
        return errorResult;
    }
}
exports.QueryTransformSchemaPipe = QueryTransformSchemaPipe;
//# sourceMappingURL=query-transform-schema.pipe.js.map